
export const fsProps = [
    'constants',
    'F_OK',
    'R_OK',
    'W_OK',
    'X_OK',
    'Stats',
];

export const fsSyncMethods = [
    'renameSync',
    'ftruncateSync',
    'truncateSync',
    'chownSync',
    'fchownSync',
    'lchownSync',
    'chmodSync',
    'fchmodSync',
    'lchmodSync',
    'statSync',
    'lstatSync',
    'fstatSync',
    'linkSync',
    'symlinkSync',
    'readlinkSync',
    'realpathSync',
    'unlinkSync',
    'rmdirSync',
    'mkdirSync',
    'mkdirpSync',
    'readdirSync',
    'closeSync',
    'openSync',
    'utimesSync',
    'futimesSync',
    'fsyncSync',
    'writeSync',
    'readSync',
    'readFileSync',
    'writeFileSync',
    'appendFileSync',
    'existsSync',
    'accessSync',
    'fdatasyncSync',
    'mkdtempSync',
    'copyFileSync',

    'createReadStream',
    'createWriteStream',
];

export const fsAsyncMethods = [
    'rename',
    'ftruncate',
    'truncate',
    'chown',
    'fchown',
    'lchown',
    'chmod',
    'fchmod',
    'lchmod',
    'stat',
    'lstat',
    'fstat',
    'link',
    'symlink',
    'readlink',
    'realpath',
    'unlink',
    'rmdir',
    'mkdir',
    'mkdirp',
    'readdir',
    'close',
    'open',
    'utimes',
    'futimes',
    'fsync',
    'write',
    'read',
    'readFile',
    'writeFile',
    'appendFile',
    'exists',
    'access',
    'fdatasync',
    'mkdtemp',
    'copyFile',

    'watchFile',
    'unwatchFile',
    'watch',
];
